/************************************************************************
*									*
*  A solution for Assignment 3, CSC108S, Spring 1998.			*
*									*
*  This solution checks for many input errors, but it does not try to	*
*  handle IOExcpetions, since the default action seems to be about as	*
*  good as anything else in this case.					*
*									*
*					Ken Jackson, 7 March 1998.	*
*									*
************************************************************************/


import java.io.* ;

class CourseManager {

   public static void main (String[] args) throws IOException {


      /******************************************************************
      *									*
      * This method processes the commands listed in assignment 3.  It	*
      * frist checks that the first command entered is MaxCourseSize.	*
      * If not, it aborts the computation.  Otherwise it enters a	*
      * do-while loop, processing one command each iteration.		*
      *									*
      * This method intantiates the object course_list of the		*
      * CourseList class and relies on the methods in that class to	*
      * process the commands.						*
      *									*
      * Note:  I have used commands as literal strings in the program	*
      * below, rather than using constants, because (except for		*
      * MaxCourseSize) the literal strings occur only once.  You could	*
      * use constants in this case, but it seems like "over-kill" to	*
      * me.								*
      *									*
      *	Note also that this method contains fewer comments that normal  *
      * because the output statements act as comments in many cases.    *
      *									*
      ******************************************************************/


      DataInputStream stdin = new DataInputStream (System.in) ;
   

      // Process the first command, which must be MaxCourseSize,

      System.out.println 
	 ( "\nType your first command (which must be MaxCourseSize)." ) ;

      // Read a line and trim white space from both ends.
      String command = stdin.readLine().trim() ;  

      if ( ! command.equals("MaxCourseSize") ) {

        // Since the first command was not MaxCourseSize,
        // print an error message and abort the computation.

        System.out.println 
	   ( "\nThe first command was not MaxCourseSize." 
	   + "\nIt was: " + command 
           + "\nProgram terminating abnormally. \n\n\n" ) ; 

        System.exit(0) ;

      } // end if


      // The first cammand was MaxCourseSize.  So now ask for
      // the maximum number of students allowed in the course.

      System.out.println 
	  ( "\nType the maximum number of students allowed in the course." ) ;
   
      ConvertToInt size = new ConvertToInt ( stdin.readLine() ) ;

      if ( size.conversionError() ) {
           
         // Since the string just read is not a valid int, 
         // print an error message and abort the computation.

         System.out.println
            ( "\nYou were supposed to type an int, but you typed "
            + "\n     " + size.stringValue() 
            + "\nProgram terminating abnormally. \n\n\n" ) ; 
         System.exit(0) ;

      } // end if

      else if ( size.intValue() <=  0 ) {

         // Since the integer just read is <= 0, 
         // print an error message and abort the computation.

         System.out.println 
	    ( "\nThe MaxCourseSize command was followed by a non-positive "
	    + "integer: " + size.intValue() 
            + "\nProgram terminating abnormally. \n\n\n" ) ; 

         System.exit(0) ;

      } // end if



      // The first command was MaxCourseSize followed by a
      // positive integer (contained in the object size).  
      // So initialize the course_list to that size.

      CourseList course_list = new CourseList ( size.intValue() ) ;



      // Start the do-while loop which processes one command each iteration.

      do {
      
	 // Get the next command

         System.out.println ( "\nType your next command." ) ;

         // Read a line and trim white space from both ends.
         command = stdin.readLine().trim() ;

	 // Determine which command was typed and then process it.

         if ( command.equals("MaxCourseSize") ) 

            System.out.println 
	       ( "\nYou typed the command MaxCourseSize, but you cannot"
	       + "\nchange the course size now."
               + "\nCommand ignored." ) ;

         else if ( command.equals("AddStudent") ) {

	    System.out.println ( "\nType the student's name." ) ;

            // Read a line and trim white space from both ends.
            String student_name = stdin.readLine().trim() ;

	    System.out.println ( "\nType the student's number." ) ;

            // Read a line and trim white space from both ends.
            String student_number = stdin.readLine().trim() ;

            if ( Student.invalid_student_number(student_number) )

               System.out.println 
		  ( "\nYou tried to add the student:"
                  + "\n   " + student_name 
                  + "\n   " + student_number 
                  + "\nto the course, but the student number is not valid."
                  + "\nCommand and data ignored." ) ;

            else if ( course_list.found_student_number(student_number) ) 

               System.out.println 
		  ( "\nYou tried to add the student:"
                  + "\n   " + student_name 
                  + "\n   " + student_number 
                  + "\nto the course, but there is already a student with"
		  + "\nthat number in the course."
                  + "\nCommand and data ignored." ) ;

            else if (course_list.full() )

               System.out.println 
		  ( "\nYou tried to add the student:"
                  + "\n   " + student_name 
                  + "\n   " + student_number 
                  + "\nto the course, but the course is already full."
                  + "\nCommand and data ignored." ) ;

            else

                 course_list.add_student ( student_name, student_number ) ;

         } // end AddStudent command
  
	 else if ( command.equals("DeleteStudent") ) {

            System.out.println ( "\nType the student's number." ) ;

            // Read a line and trim white space from both ends.
            String student_number = stdin.readLine().trim() ;

            if ( Student.invalid_student_number(student_number) )

               System.out.println
                  ( "\nYou tried to delete the student with student number"
                  + "\n   " + student_number 
                  + "\nfrom the course, but that student number is not valid."
                  + "\nCommand and data ignored." ) ;

            else if ( ! course_list.found_student_number(student_number) )

               System.out.println
                  ( "\nYou tried to delete the student with student number"
                  + "\n   " + student_number 
                  + "\nfrom the course, but there is no student with"
                  + "\nthat number in the course."
                  + "\nCommand and data ignored." ) ;

            else

                 course_list.delete_student ( student_number ) ;

         } // end DeleteStudent command

	 else if ( command.equals("AddAssignmentMark") ) {

            System.out.println ( "\nType the student's number." ) ;

            // Read a line and trim white space from both ends.
            String student_number = stdin.readLine().trim() ;

            System.out.println
               ( "\nType an integer in the range "
               + Student.get_min_assignment_mark_index () 
               + " to "
               + Student.get_max_assignment_mark_index () 
               + "\nindicating which assignment mark this is." ) ;

            ConvertToInt which_assignment = new ConvertToInt 
                 ( stdin.readLine() ) ;

            System.out.println
               ( "\nType the assignment mark (which must be an integer "
               +  "in the range "
               + Student.get_min_assignment_mark () 
               + " to "
               + Student.get_max_assignment_mark () 
               + ")." ) ;

            ConvertToInt assignment_mark = new ConvertToInt 
                 ( stdin.readLine() ) ;

            if ( Student.invalid_student_number(student_number) )

               System.out.println
                  ( "\nYou tried to add a mark of "
                  + assignment_mark.stringValue()
                  + " for assignment "
                  + which_assignment.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut the student number is not valid."
                  + "\nCommand and data ignored." ) ;

            else if ( ! course_list.found_student_number(student_number) )

               System.out.println
                  ( "\nYou tried to add a mark of "
                  + assignment_mark.stringValue()
                  + " for assignment "
                  + which_assignment.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut there is no student with that number "
                  + "in the course."
                  + "\nCommand and data ignored." ) ;

            else if ( which_assignment.conversionError() )

               System.out.println
                  ( "\nYou tried to add a mark of "
                  + assignment_mark.stringValue()
                  + " for assignment "
                  + which_assignment.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut " + which_assignment.stringValue()
                  + " is not a valid integer."
                  + "\nCommand and data ignored." ) ;

            else if ( which_assignment.intValue()
                            < Student.get_min_assignment_mark_index ()
                   || which_assignment.intValue()
                            > Student.get_max_assignment_mark_index () )

                System.out.println
                  ( "\nYou tried to add a mark of "
                  + assignment_mark.stringValue() 
                  + " for assignment "
                  + which_assignment.intValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut "
                  + which_assignment.intValue()
                  + " is not in the required range "
                  + Student.get_min_assignment_mark_index () 
                  + " to "
                  + Student.get_max_assignment_mark_index () + "." 
                  + "\nCommand and data ignored." ) ;

            else if ( assignment_mark.conversionError() )

               System.out.println
                  ( "\nYou tried to add a mark of "
                  + assignment_mark.stringValue()
                  + " for assignment "
                  + which_assignment.intValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut " + assignment_mark.stringValue()
                  + " is not a valid integer."
                  + "\nCommand and data ignored." ) ;

            else if ( assignment_mark.intValue() 
                            < Student.get_min_assignment_mark ()
                   || assignment_mark.intValue() 
                            > Student.get_max_assignment_mark () )

                System.out.println
                  ( "\nYou tried to add a mark of "
                  + assignment_mark.intValue() 
                  + " for assignment "
                  + which_assignment.intValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut "
                  + assignment_mark.intValue() 
                  + " is not in the required range "
                  + Student.get_min_assignment_mark () 
                  + " to "
                  + Student.get_max_assignment_mark () + "." 
                  + "\nCommand and data ignored." ) ;

            else 

               course_list.add_assignment_mark 
                  ( student_number, 
                    which_assignment.intValue(), 
                    assignment_mark.intValue() ) ;

         } // end AddAssignmentMark command

	 else if ( command.equals("AddProjectMark") ) {

            System.out.println ( "\nType the student's number." ) ;

            // Read a line and trim white space from both ends.
            String student_number = stdin.readLine().trim() ;

            System.out.println
               ( "\nType the project mark (which must be an integer "
               +  "in the range "
               + Student.get_min_project_mark () 
               + " to "
               + Student.get_max_project_mark () 
               + ")." ) ;

            ConvertToInt project_mark = new ConvertToInt 
                 ( stdin.readLine() ) ;

            if ( Student.invalid_student_number(student_number) )

               System.out.println
                  ( "\nYou tried to add a project mark of "
                  + project_mark.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut the student number is not valid."
                  + "\nCommand and data ignored." ) ;

            else if ( ! course_list.found_student_number(student_number) )

               System.out.println
                  ( "\nYou tried to add a project mark of "
                  + project_mark.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut there is no student with that number "
                  + "in the course."
                  + "\nCommand and data ignored." ) ;

            else if ( project_mark.conversionError() )

               System.out.println
                  ( "\nYou tried to add a project mark of "
                  + project_mark.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut " + project_mark.stringValue()
                  + " is not a valid integer."
                  + "\nCommand and data ignored." ) ;

            else if ( project_mark.intValue() 
                            < Student.get_min_project_mark ()
                   || project_mark.intValue() 
                            > Student.get_max_project_mark () )

                System.out.println
                  ( "\nYou tried to add a project mark of "
                  + project_mark.intValue() 
                  + " for the student "
                  + student_number + ","
                  + "\nbut "
                  + project_mark.intValue() 
                  + " is not in the required range "
                  + Student.get_min_project_mark () 
                  + " to "
                  + Student.get_max_project_mark () + "." 
                  + "\nCommand and data ignored." ) ;

            else 

               course_list.add_project_mark 
                  ( student_number, 
                    project_mark.intValue() ) ;

         } // end AddProjectMark command

	 else if ( command.equals("AddTermTestMark") ) {

            System.out.println ( "\nType the student's number." ) ;

            // Read a line and trim white space from both ends.
            String student_number = stdin.readLine().trim() ;

            System.out.println
               ( "\nType the term test mark (which must be an integer "
               +  "in the range "
               + Student.get_min_term_test_mark () 
               + " to "
               + Student.get_max_term_test_mark () 
               + ")." ) ;

            ConvertToInt term_test_mark = new ConvertToInt 
                 ( stdin.readLine() ) ;

            if ( Student.invalid_student_number(student_number) )

               System.out.println
                  ( "\nYou tried to add a term test mark of "
                  + term_test_mark.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut the student number is not valid."
                  + "\nCommand and data ignored." ) ;

            else if ( ! course_list.found_student_number(student_number) )

               System.out.println
                  ( "\nYou tried to add a term test mark of "
                  + term_test_mark.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut there is no student with that number "
                  + "in the course."
                  + "\nCommand and data ignored." ) ;

            else if ( term_test_mark.conversionError() )

               System.out.println
                  ( "\nYou tried to add a term test mark of "
                  + term_test_mark.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut " + term_test_mark.stringValue()
                  + " is not a valid integer."
                  + "\nCommand and data ignored." ) ;

            else if ( term_test_mark.intValue() 
                            < Student.get_min_term_test_mark ()
                   || term_test_mark.intValue() 
                            > Student.get_max_term_test_mark () )

                System.out.println
                  ( "\nYou tried to add a term test mark of "
                  + term_test_mark.intValue() 
                  + " for the student "
                  + student_number + ","
                  + "\nbut "
                  + term_test_mark.intValue() 
                  + " is not in the required range "
                  + Student.get_min_term_test_mark () 
                  + " to "
                  + Student.get_max_term_test_mark () + "." 
                  + "\nCommand and data ignored." ) ;

            else 

               course_list.add_term_test_mark 
                  ( student_number, 
                    term_test_mark.intValue() ) ;

         }  // end AddTermTestMark command

	 else if ( command.equals("AddExamMark") ) {

            System.out.println ( "\nType the student's number." ) ;

            // Read a line and trim white space from both ends.
            String student_number = stdin.readLine().trim() ;

            System.out.println
               ( "\nType the exam mark (which must be an integer "
               +  "in the range "
               + Student.get_min_exam_mark () 
               + " to "
               + Student.get_max_exam_mark () 
               + ")." ) ;

            ConvertToInt exam_mark = new ConvertToInt 
                 ( stdin.readLine() ) ;

            if ( Student.invalid_student_number(student_number) )

               System.out.println
                  ( "\nYou tried to add an exam mark of "
                  + exam_mark.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut the student number is not valid."
                  + "\nCommand and data ignored." ) ;

            else if ( ! course_list.found_student_number(student_number) )

               System.out.println
                  ( "\nYou tried to add an exam mark of "
                  + exam_mark.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut there is no student with that number "
                  + "in the course."
                  + "\nCommand and data ignored." ) ;

            else if ( exam_mark.conversionError() )

               System.out.println
                  ( "\nYou tried to add an exam mark of "
                  + exam_mark.stringValue()
                  + " for the student "
                  + student_number + ","
                  + "\nbut " + exam_mark.stringValue()
                  + " is not a valid integer."
                  + "\nCommand and data ignored." ) ;

            else if ( exam_mark.intValue() 
                            < Student.get_min_exam_mark ()
                   || exam_mark.intValue() 
                            > Student.get_max_exam_mark () )

                System.out.println
                  ( "\nYou tried to add an exam mark of "
                  + exam_mark.intValue() 
                  + " for the student "
                  + student_number + ","
                  + "\nbut "
                  + exam_mark.intValue() 
                  + " is not in the required range "
                  + Student.get_min_exam_mark () 
                  + " to "
                  + Student.get_max_exam_mark () + "." 
                  + "\nCommand and data ignored." ) ;

            else 

               course_list.add_exam_mark 
                  ( student_number, 
                    exam_mark.intValue() ) ;

         } // end AddExamMark command

	 else if ( command.equals("ComputeFinalGrades") ) 

              course_list.compute_final_grades() ;

	 else if ( command.equals("ReportGrades") ) 

              course_list.report_grades() ;

	 else if ( command.equals("ReportStats") ) 

              course_list.report_stats() ;

	 else if ( command.equals("Stop") ) 

            System.out.println
	       ( "\nProgram terminating normally. \n\n\n" ) ;

         else

            System.out.println 
	       ( "\nYou typed an invalid command:  "  + command 
               + "\nCommand ignored." ) ;
       
      } while ( ! command.equals("Stop") ) ;

      
   } // end main

} // end CourseManager 
